using System;
using System.ComponentModel;
using System.Globalization;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.Diagnostics;
using System.Collections;
using System.Reflection;
using System.Xml.Serialization;
using System.Drawing;

using Syncfusion.Windows.Forms.Diagram;
using Symcon.LightRidge.BusinessObjects.TypeConverters;


namespace Symcon.Lightridge.LRRUI.Controls
{
	[Serializable()]
	public class Aggregates : Symbol
	{
		#region Constructor
		/// <summary>
		/// Default constructor.
		/// </summary>
		public Aggregates()
		{
			SetPropertyTable();
			this.ID=Guid.NewGuid().ToString();
		//	CreateLabels();
			ArryIndexedProps = new ArrayList();

		
			for(int i = 0;i<4;i++)
			{
				ArryIndexedProps.Add("Null");
			}
			SymbolsUtils.CreateLabels(this);
		}


		/// <summary>
		/// Copy constructor.
		/// </summary>
		/// <param name="src"></param>
		public Aggregates(Aggregates src) : base(src)
		{
			try
			{
				Aggregates sourceAggs = src as Aggregates;
				Aggregates targetAggs = this;

				targetAggs.PT = null;
				targetAggs.MasterPlantItem = null;
				targetAggs.PlantItem = null;

				targetAggs.FillStyle.AlphaFactor = sourceAggs.FillStyle.AlphaFactor;
				targetAggs.FillStyle.Color = sourceAggs.FillStyle.Color;
				targetAggs.FillStyle.GradientAngle = sourceAggs.FillStyle.GradientAngle;
				targetAggs.FillStyle.GradientBounds = sourceAggs.FillStyle.GradientBounds;
				targetAggs.FillStyle.GradientStartColor = sourceAggs.FillStyle.GradientStartColor;
				targetAggs.FillStyle.Texture = sourceAggs.FillStyle.Texture;
				targetAggs.FillStyle.TextureWrapMode = sourceAggs.FillStyle.TextureWrapMode;
				targetAggs.FillStyle.Type = sourceAggs.FillStyle.Type;

				targetAggs.EditStyle.AllowMove = sourceAggs.EditStyle.AllowMove;
				targetAggs.EditStyle.AllowResize = sourceAggs.EditStyle.AllowResize;
				targetAggs.EditStyle.AllowRotate = sourceAggs.EditStyle.AllowRotate;
				targetAggs.EditStyle.AllowSelect = sourceAggs.EditStyle.AllowSelect;
				targetAggs.EditStyle.AllowVertexEdit = sourceAggs.EditStyle.AllowVertexEdit;
				targetAggs.EditStyle.DefaultHandleEditMode = sourceAggs.EditStyle.DefaultHandleEditMode;

				targetAggs.LineStyle.DashCap = sourceAggs.LineStyle.DashCap;
				targetAggs.LineStyle.DashOffset = sourceAggs.LineStyle.DashOffset;
				targetAggs.LineStyle.DashStyle = sourceAggs.LineStyle.DashStyle;
				targetAggs.LineStyle.EndCap = sourceAggs.LineStyle.EndCap;
				targetAggs.LineStyle.LineColor = sourceAggs.LineStyle.LineColor;
				targetAggs.LineStyle.LineJoin = sourceAggs.LineStyle.LineJoin;
				targetAggs.LineStyle.LineWidth = sourceAggs.LineStyle.LineWidth;
				targetAggs.LineStyle.MiterLimit = sourceAggs.LineStyle.MiterLimit;

				targetAggs.Width = sourceAggs.Width;
				targetAggs.Height = sourceAggs.Height;
				targetAggs.Length = sourceAggs.Length;

				targetAggs.Visible = sourceAggs.Visible;

				targetAggs.AutoHidePorts = sourceAggs.AutoHidePorts;
				targetAggs.BorderColor = sourceAggs.BorderColor;

				targetAggs.BorderWidth = sourceAggs.BorderWidth;
				targetAggs.Color = sourceAggs.Color;

				targetAggs.Transparency = sourceAggs.Transparency;

				targetAggs.ArryIndexedProps = null;
				targetAggs.ArryIndexedProps = sourceAggs.ArryIndexedProps.Clone() as ArrayList;
				for (int i = 0; i < 4; i++)
				{
					targetAggs.ArryIndexedProps[i] = "Null";
				}

				SetPropertyTable();
			}
			catch (Exception ex)
			{

			}
		}

		/// <summary>
		/// Serialization constructor for the MySymbol class.
		/// </summary>
		/// <param name="info">Serialization state information</param>
		/// <param name="context">Streaming context information</param>
		protected Aggregates(SerializationInfo info, StreamingContext context) : base(info, context)
		{
			// The Serialization constructor is invoked during deserialization or during a drag & drop operation.
			// If the MySymbol type has serializable members, then initialize them with the serialized data 
			// obtained from the SerializationInfo param
		}

		// Override SymbolBase.GetObjectData() and populate the SerializationInfo param 
		// with the data (if any) that belongs to the MySymbol type . This data will be 
		// serialized as a part of the Symbol object.

		#endregion

		#region Overrides
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
		}

		/// <summary>
		/// Creates a new object that is a copy of the current instance.
		/// </summary>
		/// <returns>Copy of the object this method is invoked against</returns>
		public override object Clone()
		{
			return new Aggregates(this);
		}

		#endregion

		#region	Public Properties 
		
		/// <summary>
		/// Link to Navigate to when symbol is double-clicked
		/// </summary>
		[Browsable(true),TypeConverter(typeof(LinkTypeConverter)),ReadOnly(false)]
		public Symcon.LightRidge.BusinessObjects.GraphicalLink Link
		{
			get
			{
				object value = this.GetPropertyValue("Link");
				if (value != null && value.GetType() != typeof(string))
				{
					return (Symcon.LightRidge.BusinessObjects.GraphicalLink) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("Link", value);
				this.Labels["Link"].Text = Link.Name;
			}
		}



		[Browsable(false),ReadOnly(false)]
		public Color Color
		{
			get
			{
				
				return this.FillStyle.Color;
				//				object value = this.GetPropertyValue("FillStyle.Color");
				//				if (value != null)
				//				{
				//					return (Color) value;
				//				}
				//				return base.FillStyle.Color;
			}
			set
			{
				//	this.LineStyle.LineColor = value;;
				this.FillStyle.Color = value;
				//this.SetPropertyValue("FillStyle.Color", value);
			}
		}


		[Browsable(false),ReadOnly(false)]
		public Color BorderColor
		{
			get
			{
				return this.LineStyle.LineColor;
			}
			set
			{
				this.LineStyle.LineColor = value;
			}
		}


		[Browsable(true),ReadOnly(false)]
		public new LineStyle LineStyle
		{
			get
			{
				return base.LineStyle;
			}
		}

		[Browsable(true),ReadOnly(false)]
		public new FillStyle FillStyle
		{
			get
			{
				return base.FillStyle;
			}
		}



		
		[Browsable(true),ReadOnly(false)]
		public float BorderWidth
		{
			get
			{
				return this.LineStyle.LineWidth;
			}
			set
			{
				this.LineStyle.LineWidth = value;
			}
		}

		[Browsable(true),ReadOnly(false)]
		public bool LockAspectRatio
		{
			get
			{
				object value = this.GetPropertyValue("LockAspectRatio");
				if (value != null)
				{
					return (bool) value;
				}
				return true;
			}
			set
			{
				this.SetPropertyValue("LockAspectRatio", value);
			}
		}



		
		[Browsable(true),ReadOnly(false)]
		public int Transparency
		{
			get
			{
				return this.FillStyle.AlphaFactor;
			}
			set
			{
				this.FillStyle.AlphaFactor = value;
			}
		}


		/// <summary>
		/// Link Id
		/// </summary>
		[Browsable(false),ReadOnly(false)]
		public string LinkID
		{
			get
			{
				object value = this.GetPropertyValue("LinkID");
				if (value != null)
				{
					return (string) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("LinkID", value);
			}
		}


		/// <summary>
		/// The Width of Symbol
		/// </summary>
		[Browsable(true)]
		public float Length
		{
			get
			{
				return (float) base.Width;
			}
			set
			{
                if (value == 0)
                {
                    value = .01f;
                }
				base.Width = value;
			}
		}

		/// <summary>
		/// The Width of Symbol
		/// </summary>
		[Browsable(true)]
		public new float Height
		{
			get
			{
				return (float) base.Height;
			}
			set
			{
                if (value == 0)
                {
                    value = .01f;
                }
				base.Height = value;
			}
		}



		/// <summary>
		/// The X Cordinate of Symbol w.r.t the Canvas
		/// </summary>
		[Browsable(true)]
		public new float X
		{
			get
			{
				return base.X;
			}
			set
			{
				base.X = value;
			}
		}


		/// <summary>
		/// The Y Cordinate of Symbol w.r.t the Canvas
		/// </summary>
		[Browsable(true)]
		public new float Y
		{
			get
			{
				return base.Y;
			}
			set
			{
				base.Y = value;
			}
		}



		/// <summary>
		/// Set of Labels 
		/// </summary>
		[Browsable(false)]
		public LabelCollection BoundProperties
		{
			get
			{
				return base.Labels;
			}
		}



		/// <summary>
		/// The Type of Master Plant Item
		/// Associated with this symbol
		/// </summary>
		[Browsable(true),TypeConverter(typeof(SymbolTypeConverter)), 
		CategoryAttribute("Binding Information")]
		public Symcon.LightRidge.BusinessObjects.MasterPlantItem MasterPlantItem
		{
			get
			{
				object value = this.GetPropertyValue("MasterPlantItem");
				if (value != null)
				{
					return (Symcon.LightRidge.BusinessObjects.MasterPlantItem) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("MasterPlantItem", value);
				this.SetPropertyValue("PlantItem", null);
			}
		}


		/// <summary>
		/// The Type of Master Plant Item
		/// Associated with this symbol
		/// </summary>
		[Browsable(false),TypeConverter(typeof(SymbolTypeConverter)), 
		CategoryAttribute("Binding Information")]
		public string ItemType
		{
			get
			{
				object value = this.GetPropertyValue("ItemType");
				if (value != null)
				{
					return (string) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("ItemType", value);
				this.SetPropertyValue("SiteItem", null);
			}
		}


		/// <summary>
		/// The Type of Plant Item
		/// Associated with this symbol
		/// </summary>
		[Browsable(true),TypeConverter(typeof(ObjectNameConverter)), 
		CategoryAttribute("Binding Information")]
		public Symcon.LightRidge.BusinessObjects.SiteItem PlantItem
		{
			get
			{
				object value = this.GetPropertyValue("PlantItem");
				if (value != null)
				{
					return (Symcon.LightRidge.BusinessObjects.SiteItem) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("PlantItem", value);
			}
		}



		/// <summary>
		/// The Type of Plant Item
		/// Associated with this symbol
		/// </summary>
		[Browsable(false),TypeConverter(typeof(ObjectNameConverter)), 
		CategoryAttribute("Binding Information")]
		public string SiteItem
		{
			get
			{
				object value = this.GetPropertyValue("SiteItem");
				if (value != null)
				{
					return (string) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("SiteItem", value);
			}
		}

		
		/// <summary>
		/// PropertyTable Associated With the symbol
		/// </summary>
		[Browsable(false)]
		public PropertyTable PT
		{
			get
			{
				object value = this.GetPropertyValue("PT");
				if (value != null)
				{
					return (PropertyTable) value;
				}
				PT = new PropertyTable();
				return null;
			}
			set
			{
				try
				{
					this.SetPropertyValue("PT", value);
				}
				catch(Exception ex)
				{
					Debug.Assert(false, ex.Message);
				}
			}
		}

		[Browsable(false), ReadOnly(false)]
		public string SymbolName 
		{
			get
			{
				object value = this.GetPropertyValue("SymbolName");
				if (value != null)
				{
					return (string) value;
				}
				return "Test";
			}
			set
			{
				try
				{
					this.SetPropertyValue("SymbolName", value);
				}
				catch(Exception ex)
				{
					Debug.Assert(false, ex.Message);
				}
			}
		}

		[Browsable(false), ReadOnly(false)]
		public string ID
		{
			get
			{
				object value = this.GetPropertyValue("ID");
				if (value != null)
				{
					return (string) value;
				}
				return null;
			}
			set
			{
				try
				{
					this.SetPropertyValue("ID", value);
				}
				catch(Exception ex)
				{
					Debug.Assert(false, ex.Message);
				}
			}
		}

		[Browsable(false),ReadOnly(false)]
		public ArrayList ArryIndexedProps
		{
			get
			{
				object value = this.GetPropertyValue("ArryIndexedProps");
				if (value != null)
				{
					return (ArrayList) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("ArryIndexedProps", value);
			}
				
		}

		#endregion

		#region Overrides

//		protected override void OnLabelsChanging(LabelCollectionEventArgs evtArgs)
//		{
//			//UpdateLabels();
//			base.OnLabelsChanging (evtArgs);
//		}
		#endregion

		#region Methods



		private void SetPropertyTable()
		{
			if(PT == null)
			{
				try
				{
					PropertyInfo[] properties =	this.GetType().GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
				
					foreach	(PropertyInfo property in properties)
					{
						BrowsableAttribute[] attr = property.GetCustomAttributes(typeof(BrowsableAttribute),false) as BrowsableAttribute[];
						if(attr.Length > 0)
							if(((BrowsableAttribute)attr[0]).Browsable == false)
								continue;
						string propname	= property.Name;
						if(property.Name == "MasterPlantItem")
						{
							//PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Select the Type of Symbol" ,this.GetPropertyValue(property.Name),"",typeof(SymbolTypeConverter),this));
							PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Lists of Master Plant Item" ,this.GetPropertyValue(property.Name),"",typeof(SymbolTypeConverter),this));
						}
						else if(property.Name == "PlantItem" )
						{
							//PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Lists the Bound objects of selected type" ,this.GetPropertyValue(property.Name),"",typeof(ObjectNameConverter),this));
							PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Lists the Plant Item objects of Selected Master Plant Item",this.GetPropertyValue(property.Name),"",typeof(ObjectNameConverter),this));
							//siteitemps = new PropertySpec("PlantItem",typeof(SiteItem),"Binding Information","Lists the Plant Item objects of Selected Master Plant Item" ,si,"",typeof(Symcon.LightRidge.BusinessObjects.TypeConverters.ObjectNameConverter),sm);
						}
						else if(property.Name == "Link")
						{
							PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Link","Select the Link for the Symbol" ,this.GetPropertyValue(property.Name),"",typeof(LinkTypeConverter),this));
						}
						else
						{
							PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"General	Properties","Sets the " + property.Name + " property of selected symbol",this.GetPropertyValue(property.Name),"","", this));
						}
					}
				}	
				catch(Exception	ex)
				{
					Debug.Assert(false,ex.Message);
				}
			}
		}

		#endregion
	}
}
