using System;
using Syncfusion.Windows.Forms.Diagram;
using System.Drawing;
using System.Runtime.Serialization;
using System.Drawing.Drawing2D;
using System.ComponentModel;


namespace Symcon.Lightridge.LRRUI.Controls.Tools
{
	[Serializable]
	public class OrthogonalLink : Link
	{
		#region
		private EndPointDecorator _endPointDecorator  = new EndPointDecorator(EndPointVisuals.ClosedArrow);
		private EndPointDecorator _startPointDecorator  = new EndPointDecorator(EndPointVisuals.ClosedArrow);
		private OrthogonalLine _line;
		#endregion
		
		public OrthogonalLink()
		{
		}

		public OrthogonalLink(PointF[] pts):base(pts)
		{
		}

		
		/// <summary>
		/// Copy constructor.
		/// </summary>
		/// <param name="src"></param>
		public OrthogonalLink(OrthogonalLink src) : base(src)
		{

		}

		/// <summary>
		/// Serialization constructor for symbols.
		/// </summary>
		/// <param name="info">Serialization state information.</param>
		/// <param name="context">Streaming context information.</param>
		protected OrthogonalLink(SerializationInfo info, StreamingContext context) : base(info, context)
		{
		}


		/// <summary>
		/// Creates a new object that is a copy of the current instance.
		/// </summary>
		/// <returns>Copy of the object this method is invoked against</returns>
		public override object Clone()
		{
			return new OrthogonalLink(this);
		}

		protected override IPoints CreateLinkShape(Syncfusion.Windows.Forms.Diagram.Link.Shapes shapeType, PointF[] pts)
		{
			Shape linkShape	= null;
			try
			{
				
				_line =	new	OrthogonalLine();
				
				linkShape =	_line;
				if(linkShape !=	null)
				{
					linkShape.SetPoints(pts);
				}

			}
			catch(Exception ex)
			{
				string msg = ex.Message;
			}
			return linkShape;
		}

		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
		}

		#region Properties

		[Category("LineCap"),Editor(typeof(Syncfusion.Windows.Forms.Diagram.EndPointDecoratorEditor),typeof(System.Drawing.Design.UITypeEditor)),TypeConverter(typeof(EndPointDecoratorConverter))]
		public EndPointDecorator EndCap
		{
			get
			{
				return this.EndPoints.LastEndPointDecorator;
			}
			set
			{
				this.EndPoints.LastEndPointDecorator = value;
				if(this.EndPoints.LastEndPointDecorator != null)
				{
					//this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = this.LineStyle.LineColor;
					this.EndPoints.LastEndPointDecorator.FillStyle.Color = this.LineStyle.LineColor;
				}
			}
		}


		[Category("LineCap"), Editor(typeof(Syncfusion.Windows.Forms.Diagram.EndPointDecoratorEditor),typeof(System.Drawing.Design.UITypeEditor)),TypeConverter(typeof(EndPointDecoratorConverter))]
		public EndPointDecorator StartCap
		{
			get
			{
				return this.EndPoints.FirstEndPointDecorator;
			}
			set
			{
				this._line.FirstEndPointDecorator = value;
				if(this.EndPoints.FirstEndPointDecorator != null)
				{
					//this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = this.LineStyle.LineColor;
					this.EndPoints.FirstEndPointDecorator.FillStyle.Color = this.LineStyle.LineColor;
				}
			}
		}



		

		[Category("LineCap"),TypeConverter(typeof(SizeConverter))]
		public Size EndCapSize
		{
			get
			{
				if(this.EndPoints.LastEndPointDecorator != null)
					return this.EndPoints.LastEndPointDecorator.Size.ToSize();
				else 
					return new Size(0,0);

			}
			set
			{
				this.EndPoints.LastEndPointDecorator.Size = value;
			}
		}


		[Category("LineCap"),DefaultValue(typeof(Size),null),TypeConverter(typeof(SizeConverter))]
		public Size StartCapSize
		{
			get
			{
				if(this.EndPoints.FirstEndPointDecorator != null)
					return this.EndPoints.FirstEndPointDecorator.Size.ToSize();
				else 
					return new Size(0,0);
			}
			set
			{
				this.EndPoints.FirstEndPointDecorator.Size = value;
			}
		}

		[Category("LineCap")]
		public FillStyle StartCapFillStyle
		{
			get
			{
				if(this.EndPoints.FirstEndPointDecorator != null)
					return this.EndPoints.FirstEndPointDecorator.FillStyle;
				else 
					return null;
			}
		}

		[Category("LineCap")]
		public FillStyle EndCapFillStyle
		{
			get
			{
				if(this.EndPoints.LastEndPointDecorator != null)
					return this.EndPoints.LastEndPointDecorator.FillStyle;
				else 
					return null;
			}
		}

		[Category("LineCap")]
		public LineStyle EndCapLineStyle
		{
			get
			{
				if(this.EndPoints.LastEndPointDecorator != null)
					return this.EndPoints.LastEndPointDecorator.LineStyle;
				else 
					return null;
			}
		}


	
		[Category("LineCap")]
		public LineStyle StartCapLineStyle
		{
			get
			{
				if(this.EndPoints.FirstEndPointDecorator != null)
					return this.EndPoints.FirstEndPointDecorator.LineStyle;
				else 
					return null;
			}
		}

		#endregion

		#region Overrides
		
		protected override void OnPropertyChanged(PropertyEventArgs evtArgs)
		{
			base.OnPropertyChanged (evtArgs);
			if(evtArgs.PropertyName == "LineColor")
			{
				if(this.EndPoints.LastEndPointDecorator != null)
				{
					this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = (Color)evtArgs.NewValue;
					this.EndPoints.LastEndPointDecorator.FillStyle.Color = (Color)evtArgs.NewValue;
				}
				if(this.EndPoints.FirstEndPointDecorator != null)
				{
					this.EndPoints.FirstEndPointDecorator.LineStyle.LineColor = (Color) evtArgs.NewValue;
					this.EndPoints.FirstEndPointDecorator.FillStyle.Color = (Color)evtArgs.NewValue;
				}
			}
			
		}

		#endregion 

	}
}
