using System;
using Syncfusion.Windows.Forms.Diagram;
using System.Drawing;
using System.Runtime.Serialization;
using System.Drawing.Drawing2D;
using System.ComponentModel;


namespace Symcon.Lightridge.LRRUI.Controls.Tools
{
    ///	<summary>
    ///	Vertical Link 
    ///	</summary>
    [Serializable()]
    public class RightAngleLink : Link
    {


        private EndPointDecorator _endPointDecorator = new EndPointDecorator(EndPointVisuals.ClosedArrow);
        private EndPointDecorator _startPointDecorator = new EndPointDecorator(EndPointVisuals.ClosedArrow);
        private PolyLine _line;
        private bool _isDrawn = true;


        public RightAngleLink(PointF[] pts)
        {
            this.Points.SetPoints(pts);
            _isDrawn = false;
        }


        /// <summary>
        /// Copy constructor.
        /// </summary>
        /// <param name="src"></param>
        public RightAngleLink(RightAngleLink src)
            : base(src)
        {

        }



        /// <summary>
        /// Serialization constructor for the MySymbol class.
        /// </summary>
        /// <param name="info">Serialization state information</param>
        /// <param name="context">Streaming context information</param>
        protected RightAngleLink(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            // The Serialization constructor is invoked during deserialization or during a drag & drop operation.
            // If the MySymbol type has serializable members, then initialize them with the serialized data 
            // obtained from the SerializationInfo param
        }

        #region Fields
        private PointF ptStartPoint;
        private PointF ptEndPoint;

        #endregion

        #region Private Methods

        #endregion

        #region Public Methods




        #endregion

        #region Properties

        [Category("LineCap"), Editor(typeof(Syncfusion.Windows.Forms.Diagram.EndPointDecoratorEditor), typeof(System.Drawing.Design.UITypeEditor)), TypeConverter(typeof(EndPointDecoratorConverter))]
        public EndPointDecorator EndCap
        {
            get
            {
                return this.EndPoints.LastEndPointDecorator;
            }
            set
            {
                this.EndPoints.LastEndPointDecorator = value;
                if (this.EndPoints.LastEndPointDecorator != null)
                {
                    //this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = this.LineStyle.LineColor;
                    this.EndPoints.LastEndPointDecorator.FillStyle.Color = this.LineStyle.LineColor;
                }
            }
        }


        [Category("LineCap"), Editor(typeof(Syncfusion.Windows.Forms.Diagram.EndPointDecoratorEditor), typeof(System.Drawing.Design.UITypeEditor)), TypeConverter(typeof(EndPointDecoratorConverter))]
        public EndPointDecorator StartCap
        {
            get
            {
                return this.EndPoints.FirstEndPointDecorator;
            }
            set
            {
                this.EndPoints.FirstEndPointDecorator = value;
                if (this.EndPoints.FirstEndPointDecorator != null)
                {
                    //this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = this.LineStyle.LineColor;
                    this.EndPoints.FirstEndPointDecorator.FillStyle.Color = this.LineStyle.LineColor;
                }
            }
        }





        [Category("LineCap"), TypeConverter(typeof(SizeConverter))]
        public Size EndCapSize
        {
            get
            {
                if (this.EndPoints.LastEndPointDecorator != null)
                    return this.EndPoints.LastEndPointDecorator.Size.ToSize();
                else
                    return new Size(0, 0);

            }
            set
            {
                this.EndPoints.LastEndPointDecorator.Size = value;
            }
        }


        [Category("LineCap"), DefaultValue(typeof(Size), null), TypeConverter(typeof(SizeConverter))]
        public Size StartCapSize
        {
            get
            {
                if (this.EndPoints.FirstEndPointDecorator != null)
                    return this.EndPoints.FirstEndPointDecorator.Size.ToSize();
                else
                    return new Size(0, 0);
            }
            set
            {
                this.EndPoints.FirstEndPointDecorator.Size = value;
            }
        }

        [Category("LineCap")]
        public FillStyle StartCapFillStyle
        {
            get
            {
                if (this.EndPoints.FirstEndPointDecorator != null)
                    return this.EndPoints.FirstEndPointDecorator.FillStyle;
                else
                    return null;
            }
        }

        [Category("LineCap")]
        public FillStyle EndCapFillStyle
        {
            get
            {
                if (this.EndPoints.LastEndPointDecorator != null)
                    return this.EndPoints.LastEndPointDecorator.FillStyle;
                else
                    return null;
            }
        }

        [Category("LineCap")]
        public LineStyle EndCapLineStyle
        {
            get
            {
                if (this.EndPoints.LastEndPointDecorator != null)
                    return this.EndPoints.LastEndPointDecorator.LineStyle;
                else
                    return null;
            }
        }



        [Category("LineCap")]
        public LineStyle StartCapLineStyle
        {
            get
            {
                if (this.EndPoints.FirstEndPointDecorator != null)
                    return this.EndPoints.FirstEndPointDecorator.LineStyle;
                else
                    return null;
            }
        }

        #endregion

        #region  Overrides

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>Copy of the object this method is invoked against</returns>
        public override object Clone()
        {
            return new RightAngleLink(this);
        }


        protected override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);
        }

        
        protected override void OnMoveVertex(VertexEventArgs evtArgs)
        {
            if (_isDrawn)
            {
                _isDrawn = false;
                
                return;
            }
            ptStartPoint = this.EndPoints.GetEndPointLocation(LineEndPoint.First);
            ptEndPoint = this.EndPoints.GetEndPointLocation(LineEndPoint.Last);

            if (this.Points.PointCount > 2)
            {
                PointF pt = ptEndPoint;
                for (int i = this.Points.PointCount; i > 0; i--)
                {
                    PointF var = this.Points.GetPoint(i - 1);
                    if (pt != var && var != ptStartPoint)
                    {
                        PointF prevPoint = this.Points.GetPoint(i - 2);
                        if ((pt.X != var.X && pt.Y != var.Y) || (prevPoint.X != var.X && prevPoint.Y != var.Y))
                        {
                            if (Math.Abs(pt.X - var.X) > Math.Abs(pt.Y - var.Y))
                            {
                                PointF newPoint = new PointF(prevPoint.X, pt.Y);
                                _isDrawn = true;
                                this.Points.SetPoint(i - 1, newPoint);
                                var = newPoint;
                            }
                            else
                            {
                                PointF newPoint = new PointF(pt.X, prevPoint.Y);
                                _isDrawn = true;
                                this.Points.SetPoint(i - 1, newPoint);
                                var = newPoint;
                            }
                        }
                    }
                    pt = var;
                }
            }
            base.OnMoveVertex(evtArgs);
        }

        protected override void OnConnectionsChangeComplete(ConnectionCollectionEventArgs evtArgs)
        {
            //_isDrawn = false;
            base.OnConnectionsChangeComplete(evtArgs);
            if (evtArgs.ChangeType == CollectionExChangeType.Insert)
            {
                //if (evtArgs.Connection.TargetPort.GetType().BaseType == typeof(LinkPort))
                //{
                //    if (evtArgs.Connection.SourcePort.Parent.GetType().BaseType == typeof(Symbol))
                //    {
                //        //Symbol smbl = evtArgs.Connection.SourcePort.Parent as Symbol;
                //    }
                //}

                if (evtArgs.Connection.TargetPort.GetType().BaseType == typeof(LinkPort))
                {
                    //if (evtArgs.Connection.TargetPort.Parent.GetType().BaseType == typeof(Symbol))
                    //{
                       // Symbol smbl = evtArgs.Connection.TargetPort.Parent as Symbol;
                        ptStartPoint = this.EndPoints.GetEndPointLocation(LineEndPoint.First);
                        ptEndPoint = this.EndPoints.GetEndPointLocation(LineEndPoint.Last);
                        if (this.Points.PointCount < 3)
                        {
                            if (ptEndPoint.X != ptStartPoint.X)
                            {
                                PointF pt;
                                if (Math.Abs(ptStartPoint.X - ptEndPoint.X) > Math.Abs(ptStartPoint.Y - ptEndPoint.Y))
                                {
                                    pt = new PointF(ptStartPoint.X, ptEndPoint.Y);
                                    this.Points.InsertPoint(1, pt);   
                                }
                                else
                                {
                                    pt = new PointF(ptStartPoint.X, ptEndPoint.Y);
                                    this.Points.InsertPoint(1, pt);
                                }
                            }
                        }
                        else if (this.Points.PointCount > 2)
                        {
                            PointF pt = ptEndPoint;
                            for (int i = this.Points.PointCount; i > 0; i--)
                            {
                                PointF var = this.Points.GetPoint(i - 1);
                                if (pt != var && var != ptStartPoint)
                                {
                                    PointF prevPoint = this.Points.GetPoint(i - 2);
                                    if (pt.X != var.X && pt.Y != var.Y)
                                    {
                                        if (Math.Abs(pt.X - var.X) > Math.Abs(pt.Y - var.Y))
                                        {
                                            PointF newPoint = new PointF(prevPoint.X, pt.Y);
                                            this.Points.SetPoint(i - 1, newPoint);
                                            var = newPoint;
                                        }
                                        else
                                        {
                                            PointF newPoint = new PointF(pt.X, prevPoint.Y);
                                            this.Points.SetPoint(i - 1, newPoint);
                                            var = newPoint;
                                        }
                                    }
                                }
                                pt = var;
                            }
                        //}
                    }
                }
            }
            //_isDrawn = true;
        }


        protected override void OnPropertyChanged(PropertyEventArgs evtArgs)
        {
            base.OnPropertyChanged(evtArgs);
            if (evtArgs.PropertyName == "LineColor")
            {
                if (this.EndPoints.LastEndPointDecorator != null)
                {
                    this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = (Color)evtArgs.NewValue;
                    this.EndPoints.LastEndPointDecorator.FillStyle.Color = (Color)evtArgs.NewValue;
                }
                if (this.EndPoints.FirstEndPointDecorator != null)
                {
                    this.EndPoints.FirstEndPointDecorator.LineStyle.LineColor = (Color)evtArgs.NewValue;
                    this.EndPoints.FirstEndPointDecorator.FillStyle.Color = (Color)evtArgs.NewValue;
                }
            }

        }

        #endregion

    }
}
