using System;
using Syncfusion.Windows.Forms.Diagram;
using System.Drawing;
using System.Runtime.Serialization;
using System.Drawing.Drawing2D;
using System.ComponentModel;


namespace Symcon.Lightridge.LRRUI.Controls.Tools
{
	///	<summary>
	///	Vertical Link 
	///	</summary>
	[Serializable()]
	public class VerticalLink :	Link
	{


		private EndPointDecorator _endPointDecorator  = new EndPointDecorator(EndPointVisuals.ClosedArrow);
		private EndPointDecorator _startPointDecorator  = new EndPointDecorator(EndPointVisuals.ClosedArrow);
		private PolyLine _line;


		public VerticalLink(PointF[] pts)
		{
		}

		
		/// <summary>
		/// Copy constructor.
		/// </summary>
		/// <param name="src"></param>
		public VerticalLink(VerticalLink src) : base(src)
		{

		}


		
		/// <summary>
		/// Serialization constructor for the MySymbol class.
		/// </summary>
		/// <param name="info">Serialization state information</param>
		/// <param name="context">Streaming context information</param>
		protected VerticalLink(SerializationInfo info, StreamingContext context) : base(info, context)
		{
			// The Serialization constructor is invoked during deserialization or during a drag & drop operation.
			// If the MySymbol type has serializable members, then initialize them with the serialized data 
			// obtained from the SerializationInfo param
		}

		#region Fields
		private PointF ptStartPoint; 
		private PointF ptEndPoint;

		#endregion

		#region Private Methods

		#endregion

		#region Public Methods



		 
		#endregion

		#region Properties

		[Category("LineCap"),Editor(typeof(Syncfusion.Windows.Forms.Diagram.EndPointDecoratorEditor),typeof(System.Drawing.Design.UITypeEditor)),TypeConverter(typeof(EndPointDecoratorConverter))]
		public EndPointDecorator EndCap
		{
			get
			{
				return this.EndPoints.LastEndPointDecorator;
			}
			set
			{
				this.EndPoints.LastEndPointDecorator = value;
				if(this.EndPoints.LastEndPointDecorator != null)
				{
					//this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = this.LineStyle.LineColor;
					this.EndPoints.LastEndPointDecorator.FillStyle.Color = this.LineStyle.LineColor;
				}
			}
		}


		[Category("LineCap"), Editor(typeof(Syncfusion.Windows.Forms.Diagram.EndPointDecoratorEditor),typeof(System.Drawing.Design.UITypeEditor)),TypeConverter(typeof(EndPointDecoratorConverter))]
		public EndPointDecorator StartCap
		{
			get
			{
				return this.EndPoints.FirstEndPointDecorator;
			}
			set
			{
				this._line.FirstEndPointDecorator = value;
				if(this.EndPoints.FirstEndPointDecorator != null)
				{
					//this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = this.LineStyle.LineColor;
					this.EndPoints.FirstEndPointDecorator.FillStyle.Color = this.LineStyle.LineColor;
				}
			}
		}



		

		[Category("LineCap"),TypeConverter(typeof(SizeConverter))]
		public Size EndCapSize
		{
			get
			{
				if(this.EndPoints.LastEndPointDecorator != null)
					return this.EndPoints.LastEndPointDecorator.Size.ToSize();
				else 
					return new Size(0,0);

			}
			set
			{
				this.EndPoints.LastEndPointDecorator.Size = value;
			}
		}


		[Category("LineCap"),DefaultValue(typeof(Size),null),TypeConverter(typeof(SizeConverter))]
		public Size StartCapSize
		{
			get
			{
				if(this.EndPoints.FirstEndPointDecorator != null)
					return this.EndPoints.FirstEndPointDecorator.Size.ToSize();
				else 
					return new Size(0,0);
			}
			set
			{
				this.EndPoints.FirstEndPointDecorator.Size = value;
			}
		}

		[Category("LineCap")]
		public FillStyle StartCapFillStyle
		{
			get
			{
				if(this.EndPoints.FirstEndPointDecorator != null)
					return this.EndPoints.FirstEndPointDecorator.FillStyle;
				else 
					return null;
			}
		}

		[Category("LineCap")]
		public FillStyle EndCapFillStyle
		{
			get
			{
				if(this.EndPoints.LastEndPointDecorator != null)
					return this.EndPoints.LastEndPointDecorator.FillStyle;
				else 
					return null;
			}
		}

		[Category("LineCap")]
		public LineStyle EndCapLineStyle
		{
			get
			{
				if(this.EndPoints.LastEndPointDecorator != null)
					return this.EndPoints.LastEndPointDecorator.LineStyle;
				else 
					return null;
			}
		}


	
		[Category("LineCap")]
		public LineStyle StartCapLineStyle
		{
			get
			{
				if(this.EndPoints.FirstEndPointDecorator != null)
					return this.EndPoints.FirstEndPointDecorator.LineStyle;
				else 
					return null;
			}
		}

		#endregion

		#region  Overrides

		/// <summary>
		/// Creates a new object that is a copy of the current instance.
		/// </summary>
		/// <returns>Copy of the object this method is invoked against</returns>
		public override object Clone()
		{
			return new VerticalLink(this);
		}


		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
		}


		protected override void OnMoveVertex(VertexEventArgs evtArgs)
		{
			ptStartPoint =  this.EndPoints.GetEndPointLocation(LineEndPoint.First);
			ptEndPoint = this.EndPoints.GetEndPointLocation(LineEndPoint.Last );
			if(ptEndPoint.X != ptStartPoint.X )
			{
				PointF pt;
				if(this.Points.PointCount <3)
				{
					pt = new PointF(ptEndPoint.X,ptStartPoint.Y);
					this.Points.InsertPoint(1,pt);
				}
				else
				{
					pt = new PointF(ptEndPoint.X,ptStartPoint.Y);
					this.Points.SetPoint(1,pt);

				}


			}
			base.OnMoveVertex (evtArgs);
		}

		protected override IPoints CreateLinkShape(Syncfusion.Windows.Forms.Diagram.Link.Shapes	shapeType, System.Drawing.PointF[] pts)
		{
			Shape linkShape	= null;
			try
			{
				
				_line =	new	PolyLine();
				
				linkShape =	_line;
				if(linkShape !=	null)
				{
					if(pts[0].X != pts[1].X)
					{
						PointF pt = new PointF(pts[1].X,pts[0].Y);
						PointF endPoint = pts[1];
						pts[1] = pt;
						pts[2] = endPoint;
					}

					linkShape.SetPoints(pts);
				}

//				this.EndPoints.LastEndPointDecorator = _endPointDecorator;
//				this.EndPoints.FirstEndPointDecorator = _startPointDecorator;
			}
			catch(Exception ex)
			{
				string msg = ex.Message;
			}
			return linkShape;
		}

		protected override void	OnConnectionsChangeComplete(ConnectionCollectionEventArgs evtArgs)
		{
			
			
			base.OnConnectionsChangeComplete (evtArgs);
			if(evtArgs.ChangeType == CollectionExChangeType.Insert)
			{
                //if (evtArgs.Connection.SourcePort.GetType().Name =="CirclePort")
                //{
                //    if(evtArgs.Connection.SourcePort.Parent.GetType().BaseType == typeof(Symbol))
                //    {
                //        Symbol smbl	= evtArgs.Connection.SourcePort.Parent as Symbol;
                //    }
                //}

				if (evtArgs.Connection.TargetPort.GetType().Name=="CirclePort")
				{
					if(evtArgs.Connection.TargetPort.Parent.GetType().BaseType == typeof(Symbol))
					{
						//Symbol smbl	= evtArgs.Connection.TargetPort.Parent as Symbol;
						ptStartPoint =  this.EndPoints.GetEndPointLocation(LineEndPoint.First);
						ptEndPoint = this.EndPoints.GetEndPointLocation(LineEndPoint.Last );
						if(ptEndPoint.X != ptStartPoint.X )
						{
							PointF pt;
							if(this.Points.PointCount <3)
							{
								pt = new PointF(ptEndPoint.X,ptStartPoint.Y);
								this.Points.InsertPoint(1,pt);
							}
							else
							{
								pt = new PointF(ptEndPoint.X,ptStartPoint.Y);
								this.Points.SetPoint(1,pt);

							}
						}
					}
				}
			}
		}

		


		protected override void OnPropertyChanged(PropertyEventArgs evtArgs)
		{
			base.OnPropertyChanged (evtArgs);
			if(evtArgs.PropertyName == "LineColor")
			{
				if(this.EndPoints.LastEndPointDecorator != null)
				{
					this.EndPoints.LastEndPointDecorator.LineStyle.LineColor = (Color)evtArgs.NewValue;
					this.EndPoints.LastEndPointDecorator.FillStyle.Color = (Color)evtArgs.NewValue;
				}
				if(this.EndPoints.FirstEndPointDecorator != null)
				{
					this.EndPoints.FirstEndPointDecorator.LineStyle.LineColor = (Color) evtArgs.NewValue;
					this.EndPoints.FirstEndPointDecorator.FillStyle.Color = (Color)evtArgs.NewValue;
				}
			}
			
		}

		#endregion

	}
}
