
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing.Design;
using System.Diagnostics;

namespace Symcon.Lightridge.LRRUI
{

	/// <summary>
	/// An extension of PropertyBag that manages a table of property values, in
	/// addition to firing events when property values are requested or set.
	/// </summary>
	/// 
	[Serializable()]
	public class PropertyTable : PropertyBag
	{
		private Hashtable propValues;
		//		private object symbol;

		/// <summary>
		/// Initializes a new instance of the PropertyTable class.
		/// </summary>
		public PropertyTable()
		{
			propValues = new Hashtable();
			
		}

		/// <summary>
		/// Gets or sets the value of the property with the specified name.
		/// <p>In C#, this property is the indexer of the PropertyTable class.</p>
		/// </summary>
		public object this[string key]
		{
			get { return propValues[key]; }
			set { propValues[key] = value; }
		}

		/// <summary>
		/// This member overrides PropertyBag.OnGetValue.
		/// </summary>
		protected override void OnGetValue(PropertySpecEventArgs e)
		{
			e.Value = propValues[e.Property.Name];
			base.OnGetValue(e);
			try
			{
				if(e.Property.ObjectRef.GetType() == typeof(string))
				{
					e.Value = e.Property.DefaultValue;
				}
				else
				{
                    if (e.Property.ObjectRef.GetType().GetProperty(e.Property.Name) != null)
                    {
                        e.Value = e.Property.ObjectRef.GetType().GetProperty(e.Property.Name).GetValue(e.Property.ObjectRef, null);
                    }
                    else if(this.Properties.Contains(e.Property.Name))
                    {
                        this.Properties.Remove(e.Property.Name);
                    }
				}

				//e.Property.ObjectRef.GetType().GetProperty(e.Property.Name).GetValue(e.Property.ObjectRef.GetType().GetProperty(e.Property.Name), null);
			}
			catch(Exception ex)
			{
				e.Value = e.Property.DefaultValue;
				Debug.Assert(true,ex.Message); 

			}
		}

		/// <summary>
		/// This member overrides PropertyBag.OnSetValue.
		/// </summary>
		protected override void OnSetValue(PropertySpecEventArgs e)
		{
			propValues[e.Property.Name] = e.Value;
			base.OnSetValue(e);
			try
			{
				e.Property.ObjectRef.GetType().GetProperty(e.Property.Name).SetValue(e.Property.ObjectRef, e.Value, null);

			}
			catch(Exception ex)
			{
				e.Property.DefaultValue = e.Value;

				Debug.Assert(true, ex.Message);
			}
		}
	}
}
